@@ -1,11 +1,7 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
  * (C) 2012-2013 by Pablo Neira Ayuso <pablo@netfilter.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published
- * by the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
  * This code has been sponsored by Sophos Astaro <http://www.sophos.com>
  */
 #include "internal.h"
@@ -37,8 +33,7 @@ struct nftnl_chain {
 	const char	*type;
 	const char	*table;
 	const char	*dev;
-	const char	**dev_array;
-	int		dev_array_len;
+	struct nftnl_str_array	dev_array;
 	uint32_t	family;
 	uint32_t	policy;
 	uint32_t	hooknum;
@@ -117,7 +112,6 @@ EXPORT_SYMBOL(nftnl_chain_free);
 void nftnl_chain_free(const struct nftnl_chain *c)
 {
 	struct nftnl_rule *r, *tmp;
-	int i;
 
 	list_for_each_entry_safe(r, tmp, &c->rule_list, head)
 		nftnl_rule_free(r);
@@ -132,12 +126,8 @@ void nftnl_chain_free(const struct nftnl_chain *c)
 		xfree(c->dev);
 	if (c->flags & (1 << NFTNL_CHAIN_USERDATA))
 		xfree(c->user.data);
-	if (c->flags & (1 << NFTNL_CHAIN_DEVICES)) {
-		for (i = 0; i < c->dev_array_len; i++)
-			xfree(c->dev_array[i]);
-
-		xfree(c->dev_array);
-	}
+	if (c->flags & (1 << NFTNL_CHAIN_DEVICES))
+		nftnl_str_array_clear((struct nftnl_str_array *)&c->dev_array);
 	xfree(c);
 }
 
@@ -150,8 +140,6 @@ bool nftnl_chain_is_set(const struct nftnl_chain *c, uint16_t attr)
 EXPORT_SYMBOL(nftnl_chain_unset);
 void nftnl_chain_unset(struct nftnl_chain *c, uint16_t attr)
 {
-	int i;
-
 	if (!(c->flags & (1 << attr)))
 		return;
 
@@ -181,9 +169,7 @@ void nftnl_chain_unset(struct nftnl_chain *c, uint16_t attr)
 		xfree(c->dev);
 		break;
 	case NFTNL_CHAIN_DEVICES:
-		for (i = 0; i < c->dev_array_len; i++)
-			xfree(c->dev_array[i]);
-		xfree(c->dev_array);
+		nftnl_str_array_clear(&c->dev_array);
 		break;
 	case NFTNL_CHAIN_USERDATA:
 		xfree(c->user.data);
@@ -212,9 +198,6 @@ EXPORT_SYMBOL(nftnl_chain_set_data);
 int nftnl_chain_set_data(struct nftnl_chain *c, uint16_t attr,
 			 const void *data, uint32_t data_len)
 {
-	const char **dev_array;
-	int len = 0, i;
-
 	nftnl_assert_attr_exists(attr, NFTNL_CHAIN_MAX);
 	nftnl_assert_validate(data, nftnl_chain_validate, attr, data_len);
 
@@ -256,24 +239,8 @@ int nftnl_chain_set_data(struct nftnl_chain *c, uint16_t attr,
 		return nftnl_set_str_attr(&c->dev, &c->flags,
 					  attr, data, data_len);
 	case NFTNL_CHAIN_DEVICES:
-		dev_array = (const char **)data;
-		while (dev_array[len] != NULL)
-			len++;
-
-		if (c->flags & (1 << NFTNL_CHAIN_DEVICES)) {
-			for (i = 0; i < c->dev_array_len; i++)
-				xfree(c->dev_array[i]);
-			xfree(c->dev_array);
-		}
-
-		c->dev_array = calloc(len + 1, sizeof(char *));
-		if (!c->dev_array)
+		if (nftnl_str_array_set(&c->dev_array, data) < 0)
 			return -1;
-
-		for (i = 0; i < len; i++)
-			c->dev_array[i] = strdup(dev_array[i]);
-
-		c->dev_array_len = len;
 		break;
 	case NFTNL_CHAIN_FLAGS:
 		memcpy(&c->chain_flags, data, sizeof(c->chain_flags));
@@ -385,7 +352,7 @@ const void *nftnl_chain_get_data(const struct nftnl_chain *c, uint16_t attr,
 		return c->dev;
 	case NFTNL_CHAIN_DEVICES:
 		*data_len = 0;
-		return &c->dev_array[0];
+		return c->dev_array.array;
 	case NFTNL_CHAIN_FLAGS:
 		*data_len = sizeof(uint32_t);
 		return &c->chain_flags;
@@ -493,11 +460,11 @@ void nftnl_chain_nlmsg_build_payload(struct nlmsghdr *nlh, const struct nftnl_ch
 		mnl_attr_put_strz(nlh, NFTA_HOOK_DEV, c->dev);
 	else if (c->flags & (1 << NFTNL_CHAIN_DEVICES)) {
 		struct nlattr *nest_dev;
+		const char *dev;
 
 		nest_dev = mnl_attr_nest_start(nlh, NFTA_HOOK_DEVS);
-		for (i = 0; i < c->dev_array_len; i++)
-			mnl_attr_put_strz(nlh, NFTA_DEVICE_NAME,
-					  c->dev_array[i]);
+		nftnl_str_array_foreach(dev, &c->dev_array, i)
+			mnl_attr_put_strz(nlh, NFTA_DEVICE_NAME, dev);
 		mnl_attr_nest_end(nlh, nest_dev);
 	}
 
@@ -664,42 +631,6 @@ static int nftnl_chain_parse_hook_cb(const struct nlattr *attr, void *data)
 	return MNL_CB_OK;
 }
 
-static int nftnl_chain_parse_devs(struct nlattr *nest, struct nftnl_chain *c)
-{
-	const char **dev_array, **tmp;
-	int len = 0, size = 8;
-	struct nlattr *attr;
-
-	dev_array = calloc(8, sizeof(char *));
-	if (!dev_array)
-		return -1;
-
-	mnl_attr_for_each_nested(attr, nest) {
-		if (mnl_attr_get_type(attr) != NFTA_DEVICE_NAME)
-			goto err;
-		dev_array[len++] = strdup(mnl_attr_get_str(attr));
-		if (len >= size) {
-			tmp = realloc(dev_array, size * 2 * sizeof(char *));
-			if (!tmp)
-				goto err;
-
-			size *= 2;
-			memset(&tmp[len], 0, (size - len) * sizeof(char *));
-			dev_array = tmp;
-		}
-	}
-
-	c->dev_array = dev_array;
-	c->dev_array_len = len;
-
-	return 0;
-err:
-	while (len--)
-		xfree(dev_array[len]);
-	xfree(dev_array);
-	return -1;
-}
-
 static int nftnl_chain_parse_hook(struct nlattr *attr, struct nftnl_chain *c)
 {
 	struct nlattr *tb[NFTA_HOOK_MAX+1] = {};
@@ -723,7 +654,7 @@ static int nftnl_chain_parse_hook(struct nlattr *attr, struct nftnl_chain *c)
 		c->flags |= (1 << NFTNL_CHAIN_DEV);
 	}
 	if (tb[NFTA_HOOK_DEVS]) {
-		ret = nftnl_chain_parse_devs(tb[NFTA_HOOK_DEVS], c);
+		ret = nftnl_parse_devs(&c->dev_array, tb[NFTA_HOOK_DEVS]);
 		if (ret < 0)
 			return -1;
 		c->flags |= (1 << NFTNL_CHAIN_DEVICES);
@@ -808,21 +739,11 @@ int nftnl_chain_nlmsg_parse(const struct nlmsghdr *nlh, struct nftnl_chain *c)
 	return ret;
 }
 
-static inline int nftnl_str2hooknum(int family, const char *hook)
-{
-	int hooknum;
-
-	for (hooknum = 0; hooknum < NF_INET_NUMHOOKS; hooknum++) {
-		if (strcmp(hook, nftnl_hooknum2str(family, hooknum)) == 0)
-			return hooknum;
-	}
-	return -1;
-}
-
 static int nftnl_chain_snprintf_default(char *buf, size_t remain,
 					const struct nftnl_chain *c)
 {
 	int ret, offset = 0, i;
+	const char *dev;
 
 	ret = snprintf(buf, remain, "%s %s %s use %u",
 		       nftnl_family2str(c->family), c->table, c->name, c->use);
@@ -854,9 +775,9 @@ static int nftnl_chain_snprintf_default(char *buf, size_t remain,
 			ret = snprintf(buf + offset, remain, " dev { ");
 			SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 
-			for (i = 0; i < c->dev_array_len; i++) {
+			nftnl_str_array_foreach(dev, &c->dev_array, i) {
 				ret = snprintf(buf + offset, remain, " %s ",
-					       c->dev_array[i]);
+					       dev);
 				SNPRINTF_BUFFER_SIZE(ret, remain, offset);
 			}
 			ret = snprintf(buf + offset, remain, " } ");
